----------------------------------------------------------------------------------
-- Company: EE Solutions LLC, www.eesoltionsllc.com
-- Engineer: A Hill
-- 
-- Create Date:    12:50:34 11/30/2006 
-- Design Name:    usb_test
-- Module Name:    usb_state_machine - Behavioral 
-- Project Name:   DLP_FPGA
-- Target Devices: XC3S100E-4TQ144A
-- Tool versions:  ISE 8.2.03i
-- Description:    Module interfaces to FTDI USB controller 
--						 Reads bytes when available from USB
--						 Sends bytes as appropriate based on commands
--                 
-- Dependencies:   IO Buffers contained in top module (demo.vhd)
--						 Commmands 30,40,50&60 require cmd_exec module
--                 State machine will hang waiting for local done
--                 if cmd_exec has an issue
--
-- Revision: 
-- 	0.01 	AH 11/30/06	File Created
--    0.02  AH 12/02/06 Simple state machine echo's USB byte sent
--		0.03	AH	12/15/06 Initial state machine structure completed
--								Command 00 (port alive) works
--    0.04  AH 12/27/06 Removed uneeded UBUSY state
--								Commands 00,10,2x all simulate correct
--		0.05	AH	12/28/06 Changed read pin value to 8 bits to account
--                      for error case
--    0.06  AH 02/14/07 Modified ID vectors for debug. Debugging
--                      w/cscope. Design is violating time between 
--                      read cycles. 
--    0.07  AH 02/26/07 Fixed cbyte4 else state. Added 100 ns wait
--                      (wait done) to pbyte, and avail2-avail5
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity usb_state_machine is
   GENERIC(	
		BOARD_ID  : std_logic_vector(7 downto 0);--:=x"12";  -- 1=initial proto
      FPGA_ID   : std_logic_vector(7 downto 0);--:=x"34";  -- 1=XC3S100E-4TQ144CES
	   DESIGN_ID : std_logic_vector(7 downto 0));--:=x"56"); -- 1=initial design
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		RXF_n : IN std_logic;
		TXE_n : IN std_logic;
		ftdi_rd : OUT std_logic;
		ftdi_wr : OUT std_logic;
		ftdi_dir : out STD_LOGIC; -- to tristates at top level
      ftdi_din : in  STD_LOGIC_VECTOR (7 downto 0); -- data from USB to local
      ftdi_dout : out  STD_LOGIC_VECTOR (7 downto 0); -- data_from local to USB
		local_ready : out  STD_LOGIC; -- data ready for local user
		local_done : in  STD_LOGIC; -- local user done with data
		local_din : in  STD_LOGIC_VECTOR (7 downto 0); -- data to USB port;
		local_dout : out  STD_LOGIC_VECTOR (7 downto 0); -- data from USB port;
		ram_addr : out STD_LOGIC_VECTOR (16 downto 0);
		ram_data : out STD_LOGIC_VECTOR (7 downto 0);
		pinget : out std_logic;
		pinset : out std_logic;
		pinclr : out std_logic;
		r_ram : out std_logic;
		w_ram : out std_logic);
end usb_state_machine;

architecture Behavioral of usb_state_machine is

   type state_type is (idle,avail1,rbyte1,cbyte1,style,pbyte,sbyte,avail2,
	rbyte2,cbyte2,drvpinl,drvpinh,getpin,getval,avail3,rbyte3,cbyte3,avail4,rbyte4,
	cbyte4,avail5,rbyte5,cbyte5,rsram,wsram); --26 states
   signal state, next_state : state_type; 

	-- set bits 0 = command byte, 1-4 = data bytes 1 - 4
   signal capture_i : std_logic_vector (4 downto 0);
   signal cdecode : std_logic_vector (3 downto 0);
   signal ftdi_rd_i,ftdi_wr_i : std_logic;  
   signal send_i,wait_start_i : std_logic;  
   signal pinget_i,pinset_i,pinclr_i : std_logic;  
   signal w_ram_i,r_ram_i,local_ready_i : std_logic;  
   --other outputs
   signal command,data_ah,addrmid,addrlow,wdata  : std_logic_vector (7 downto 0);
   signal capture : std_logic_vector(4 downto 0);
	signal wcount : std_logic_vector (1 downto 0);
   signal send, wait_start, wait_done : std_logic;

begin

   SYNC_PROC: process (clk)
   begin
      if (clk'event and clk = '1') then
         if (reset = '1') then
            state 			<= idle;
            ftdi_rd 		<= '1';
            ftdi_wr 		<= '1';
            capture 		<= "00000"; 
            send 			<= '0';
            wait_start 	<= '0';
				pinget		<= '0';
				pinset		<= '0';
				pinclr		<= '0';
				local_ready <= '0';
				r_ram       <= '0';
				w_ram       <= '0';
         else
            state 		<= next_state;
            ftdi_rd 		<= ftdi_rd_i;
            ftdi_wr 		<= ftdi_wr_i;
            capture 		<= capture_i;
            send 			<= send_i;
            wait_start 	<= wait_start_i;
				pinget		<= pinget_i;
				pinset		<= pinset_i;
				pinclr		<= pinclr_i;
				local_ready <= local_ready_i;
				r_ram       <= r_ram_i;
				w_ram       <= w_ram_i;
         end if;        
      end if;
   end process;
 
   --MOORE State Machine - Outputs based on state only
   OUTPUT_DECODE: process (state)
   begin
		case (state) is
			---------------------------------------------------------------------
			-- IDLE: Waiting for RXF_n to go low
			when idle =>
            ftdi_rd_i 		<= '1';
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '0';
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- AVAIL1: RXF_n goes low indicating byte(s) available
			when avail1 =>
            ftdi_rd_i 		<= '1';
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- RBYTE1: Wait timer done, issue read
			when rbyte1 =>
            ftdi_rd_i 		<= '0'; -- assert read to get byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- CBYTE1: Wait timer done, capture byte
			when cbyte1 =>
            ftdi_rd_i 		<= '0'; -- assert read to hold byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00001"; -- assert capture for user read 1
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- STYLE: Determine command, wait for local done
			when style =>
            ftdi_rd_i 		<= '1'; 
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '0'; 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- PBYTE: Present Byte, wait for TXE_n assertion
			when pbyte =>
            ftdi_rd_i 		<= '1'; 
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '1'; -- assert send
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter; 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- SBYTE: Send Byte, wait timer done
			when sbyte =>
            ftdi_rd_i 		<= '1'; 
            ftdi_wr_i 		<= '0'; -- assert write
            capture_i 		<= "00000"; 
            send_i 			<= '1'; -- assert send
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- AVAIL2: wait if RXF_n not low
			when avail2 =>
            ftdi_rd_i 		<= '1';
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- RBYTE2: Wait timer done, issue read
			when rbyte2 =>
            ftdi_rd_i 		<= '0'; -- assert read to get byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- CBYTE2: Wait timer done, capture byte
			when cbyte2 =>
            ftdi_rd_i 		<= '0'; -- assert read to hold byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00010"; -- assert capture for user read 1
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- DRVPINL: Assert FPGA pin low, wait for local done
			when drvpinl =>
            ftdi_rd_i 		<= '1'; 
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '0'; 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '1';
				local_ready_i  <= '1';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- DRVPINH: Assert FPGA pin low, wait for local done
			when drvpinh =>
            ftdi_rd_i 		<= '1'; 
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '0'; 
				pinget_i			<= '0';
				pinset_i			<= '1';
				pinclr_i			<= '0';
				local_ready_i  <= '1';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- GETPIN: Get FPGA pin value, wait for local done
			when getpin =>
            ftdi_rd_i 		<= '1'; 
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '0'; 
				pinget_i			<= '1';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '1';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- GETVAL: Get internal reg value
			when getval =>
            ftdi_rd_i 		<= '1'; 
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '0'; 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';--'1';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- AVAIL3: wait if RXF_n not low
			when avail3 =>
            ftdi_rd_i 		<= '1';
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- RBYTE3: Wait timer done, issue read
			when rbyte3 =>
            ftdi_rd_i 		<= '0'; -- assert read to get byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- CBYTE3: Wait timer done, capture byte
			when cbyte3 =>
            ftdi_rd_i 		<= '0'; -- assert read to hold byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00100"; -- assert capture for user read 1
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- AVAIL4: wait if RXF_n not low
			when avail4 =>
            ftdi_rd_i 		<= '1';
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- RBYTE4: Wait timer done, issue read
			when rbyte4 =>
            ftdi_rd_i 		<= '0'; -- assert read to get byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- CBYTE4: Wait timer done, capture byte
			when cbyte4 =>
            ftdi_rd_i 		<= '0'; -- assert read to hold byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "01000"; -- assert capture for user read 1
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- AVAIL5: wait if RXF_n not low
			when avail5 =>
            ftdi_rd_i 		<= '1';
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- RBYTE5: Wait timer done, issue read
			when rbyte5 =>
            ftdi_rd_i 		<= '0'; -- assert read to get byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- CBYTE5: Wait timer done, capture byte
			when cbyte5 =>
            ftdi_rd_i 		<= '0'; -- assert read to hold byte
            ftdi_wr_i 		<= '1';
            capture_i 		<= "10000"; -- assert capture for user read 1
            send_i 			<= '0';
            wait_start_i 	<= '1'; -- assert wait_start to start delay counter
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- RSRAM: Write value to SRAM, wait for local done
			when rsram =>
            ftdi_rd_i 		<= '1'; 
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '0'; 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '1';
				r_ram_i        <= '1';
				w_ram_i        <= '0';
			---------------------------------------------------------------------
			-- WSRAM: Write value to SRAM, wait for local done
			when wsram =>
            ftdi_rd_i 		<= '1'; 
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '0'; 
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '1';
				r_ram_i        <= '0';
				w_ram_i        <= '1';
			---------------------------------------------------------------------
			-- OTHERS: 
			when others =>
            ftdi_rd_i 		<= '1';
            ftdi_wr_i 		<= '1';
            capture_i 		<= "00000"; 
            send_i 			<= '0';
            wait_start_i 	<= '0';
				pinget_i			<= '0';
				pinset_i			<= '0';
				pinclr_i			<= '0';
				local_ready_i  <= '0';
				r_ram_i        <= '0';
				w_ram_i        <= '0';
      end case;      
	end process;
 
   NEXT_STATE_DECODE: process (state,RXF_n,TXE_n,wait_done,local_done,cdecode)
   begin
      --declare default state for next_state to avoid latches
      next_state <= state;  --default is to stay in current state
		case (state) is
			---------------------------------------------------------------------
			-- IDLE: Waiting for RXF_n to go low
			when idle =>
            if RXF_n = '0' then -- data ready to be read from USB
				   next_state <= avail1;
				else
				   next_state <= idle;
				end if;
			---------------------------------------------------------------------
			-- AVAIL1: RXF_n goes low indicating byte(s) available
			when avail1 =>
            if wait_done = '1' then -- wait time is complete
				   next_state <= rbyte1;
				else
				   next_state <= avail1;
				end if;
			---------------------------------------------------------------------
			-- RBYTE1: Wait timer done, issue read
			when rbyte1 =>
            if wait_done = '1' then -- wait time is complete
				   next_state <= cbyte1;
				else
				   next_state <= rbyte1;
				end if;
			---------------------------------------------------------------------
			-- CBYTE1: Wait timer done, capture byte
			when cbyte1 =>
            if wait_done = '1' then -- wait time is complete
				   next_state <= style;
				else
				   next_state <= cbyte1;
				end if;
			---------------------------------------------------------------------
			-- STYLE: Determine command, wait for local done
			when style =>
				if cdecode = X"0" then --single byte or unrecognized instruction
					next_state <= pbyte; --ubusy;
				else
					next_state <= avail2;
				end if;
			---------------------------------------------------------------------
			-- UBUSY: USER busy, wait for local done
			--when ubusy =>
         --   if local_done = '1' then -- local interface done
			--	   next_state <= pbyte;
			--	else
			--	   next_state <= ubusy;
			--	end if;
			---------------------------------------------------------------------
			-- PBYTE: Present Byte, wait for TXE_n assertion
			when pbyte =>
            if (TXE_n='0' and wait_done='1') then -- data ready to be writen to USB
				   next_state <= sbyte;
				else
				   next_state <= pbyte;
				end if;
			---------------------------------------------------------------------
			-- SBYTE: Send Byte, wait timer done
			when sbyte =>
            if wait_done = '1' then -- wait time is complete
				   next_state <= idle;
				else
				   next_state <= sbyte;
				end if;
			---------------------------------------------------------------------
			-- AVAIL2: wait if RXF_n not low
			when avail2 =>
            if (RXF_n='0' and wait_done='1') then -- data ready to be read from USB
				   next_state <= rbyte2;
				else
				   next_state <= avail2;
				end if;
			---------------------------------------------------------------------
			-- RBYTE2: Wait timer done, issue read
			when rbyte2 =>
            if wait_done = '1' then -- wait time is complete
				   next_state <= cbyte2;
				else
				   next_state <= rbyte2;
				end if;
			---------------------------------------------------------------------
			-- CBYTE2: Wait timer done, capture byte
			when cbyte2 =>
            if wait_done = '1' then -- wait time is complete
					case (cdecode) is
						when X"1" => next_state <= getval; -- read reg
						when X"2" => next_state <= pbyte; -- send loopback
						when X"3" => next_state <= pbyte; -- send comp loopback
						when X"4" => next_state <= getpin; -- read pin
						when X"6" => next_state <= drvpinl; -- clear pin (0)
						when X"7" => next_state <= drvpinh; -- set pin (1)
						when X"8" => next_state <= avail3; -- read next value
						when X"9" => next_state <= avail3; -- read next value
						when others => next_state <= idle;
					end case;
				else
				   next_state <= cbyte2;
				end if;
			---------------------------------------------------------------------
			-- DRVPINL: Clear FPGA pin as specified, wait for local done
			when drvpinl =>
            if local_done = '1' then -- local interface done
				   next_state <= pbyte;--ubusy;
				else
				   next_state <= drvpinl;
				end if;
			---------------------------------------------------------------------
			-- DRVPINH: Set FPGA pin as specified, wait for local done
			when drvpinh =>
            if local_done = '1' then -- local interface done
				   next_state <= pbyte;--ubusy;
				else
				   next_state <= drvpinh;
				end if;
			---------------------------------------------------------------------
			-- GETPIN: Read FPGA pin as specified, wait for local done
			when GETPIN =>
            if local_done = '1' then -- local interface done
				   next_state <= pbyte;--ubusy;
				else
				   next_state <= getpin;
				end if;
			---------------------------------------------------------------------
			-- GETVAL: Get reg value, wait for local done
			when GETVAL =>
            --if local_done = '1' then -- local interface done
				   next_state <= pbyte;
				--else
				--   next_state <= getval;
				--end if;
			---------------------------------------------------------------------
			-- AVAIL3: wait if RXF_n not low
			when avail3 =>
            if (RXF_n='0' and wait_done='1') then -- data ready to be read from USB
				   next_state <= rbyte3;
				else
				   next_state <= avail3;
				end if;
			---------------------------------------------------------------------
			-- RBYTE3: Wait timer done, issue read
			when rbyte3 =>
            if wait_done = '1' then -- wait time is complete
				   next_state <= cbyte3;
				else
				   next_state <= rbyte3;
				end if;
			---------------------------------------------------------------------
			-- CBYTE3: Wait timer done, capture byte
			when cbyte3 =>
            if wait_done = '1' then -- wait time is complete
				   next_state <= avail4;
				else
				   next_state <= cbyte3;
				end if;
			---------------------------------------------------------------------
			-- AVAIL4: wait if RXF_n not low
			when avail4 =>
            if (RXF_n='0' and wait_done='1') then -- data ready to be read from USB
				   next_state <= rbyte4;
				else
				   next_state <= avail4;
				end if;
			---------------------------------------------------------------------
			-- RBYTE4: Wait timer done, issue read
			when rbyte4 =>
            if wait_done = '1' then -- wait time is complete
				   next_state <= cbyte4;
				else
				   next_state <= rbyte4;
				end if;
			---------------------------------------------------------------------
			-- CBYTE4: Wait timer done, capture byte
			when cbyte4 =>
            if wait_done = '1' then -- wait time is complete
					if cdecode(0) = '1' then -- write SRAM needs one more value
						next_state <= avail5;
					else -- read sram has all required bytes
						next_state <= rsram;
					end if;
				else
				   next_state <= cbyte4;
				end if;
			---------------------------------------------------------------------
			-- RSRAM: Get RAM value, wait for local done
			when RSRAM =>
            if local_done = '1' then -- local interface done
				   next_state <= pbyte;--ubusy;
				else
				   next_state <= rsram;
				end if;
			---------------------------------------------------------------------
			-- AVAIL5: wait if RXF_n not low
			when avail5 =>
            if (RXF_n='0' and wait_done='1') then -- data ready to be read from USB
				   next_state <= rbyte5;
				else
				   next_state <= avail5;
				end if;
			---------------------------------------------------------------------
			-- RBYTE5: Wait timer done, issue read
			when rbyte5 =>
            if wait_done = '1' then -- wait time is complete
				   next_state <= cbyte5;
				else
				   next_state <= rbyte5;
				end if;
			---------------------------------------------------------------------
			-- CBYTE5: Wait timer done, capture byte
			when cbyte5 =>
            if wait_done = '1' then -- wait time is complete
					next_state <= wsram;
				else
				   next_state <= cbyte5;
				end if;
			---------------------------------------------------------------------
			-- WSRAM: Write value to SRAM, wait for local done
			when WSRAM =>
            if local_done = '1' then -- local interface done
				   next_state <= pbyte;--ubusy;
				else
				   next_state <= wsram;
				end if;
			---------------------------------------------------------------------
			-- OTHERS: 
			when others =>
				   next_state <= idle;	
      end case;      
   end process;

   SYNC_CAPTURE: process (clk)
   begin
      if (clk'event and clk = '1') then
         if (reset = '1') then
				command  <= X"00";
				data_ah  <= X"00";
				addrmid  <= X"00";
				addrlow  <= X"00";
				wdata    <= X"00";
				ram_addr <= (others => '0');
				ram_data <= (others => '0');
			else
				case (capture) is
					when "00001" => command <= ftdi_din;
					when "00010" => data_ah <= ftdi_din;
					when "00100" => addrmid <= ftdi_din;
					when "01000" => addrlow <= ftdi_din;
					when "10000" => wdata   <= ftdi_din;
					when others =>  command <= command; --X"00";
										 data_ah <= data_ah; --X"00";
										 addrmid <= addrmid; --X"00";
										 addrlow <= addrlow; --X"00";
										 wdata   <= wdata;   --X"00";
				end case;
				ram_addr <= data_ah(0) & addrmid & addrlow;
				ram_data <= wdata;
			end if;
		end if;
	end process;

   SYNC_CMD_DEC: process (clk)
   begin
      if (clk'event and clk = '1') then
         if (reset = '1') then
				cdecode <= X"0";
			else
			   case (command) is
				   when  X"00" => cdecode <= "0000"; -- 1 byte - port alive
				   when  X"10" => cdecode <= "0001"; -- 2 byte - return register
				   when  X"20" => cdecode <= "0010"; --          loopback byte
				   when  X"21" => cdecode <= "0011"; --          loopback 2s comp
				   when  X"30" => cdecode <= "0100"; --          read pin
				   when  X"40" => cdecode <= "0110"; --          assert pin high
				   when  X"41" => cdecode <= "0111"; --          assert pin low
				   when  X"50" => cdecode <= "1000"; -- 4 byte - read sram
				   when  X"60" => cdecode <= "1001"; -- 5 byte - write sram
				   when others => cdecode <= "0000";
				end case;
			end if;
		end if;
	end process;

   SYNC_DATAOUT: process (clk)
   begin
      if (clk'event and clk = '1') then
         if (reset = '1') then
				local_dout <= X"00";
				ftdi_dout  <= X"00";
			else
				--if (local_done = '1') then
					case (command) is
					   -- 00h = Port Alive Query
						when  X"00" => local_dout <= X"55";
										   ftdi_dout  <= X"55";
						-- 10h = Return Register Specified					
						when  X"10" => case (data_ah) is
											   when X"00" => local_dout <= BOARD_ID; 
																  ftdi_dout <= BOARD_ID; 
											   when X"01" => local_dout <= FPGA_ID; 
												              ftdi_dout <= FPGA_ID; 
											   when X"02" => local_dout <= DESIGN_ID; 
												              ftdi_dout <= DESIGN_ID;
											   when others => local_dout <= X"E0"; -- invalid
                                                   ftdi_dout <= X"E0";
										   end case;	
						-- 20h = Loopback next byte
						when  X"20" => local_dout <= data_ah;
                                 ftdi_dout <= data_ah;
						-- 21h = Loopback byte inverted
						when  X"21" => local_dout <= NOT(data_ah);
                                 ftdi_dout <= NOT(data_ah);
						-- 30h = Read Pin
						when  X"30" => local_dout <= data_ah; -- pin location
                                 ftdi_dout <= local_din; -- pin value
						-- 40h = Assert Pin Low					
						when  X"40" => local_dout <= data_ah; -- pin location
                                 ftdi_dout <= X"40";
						-- 41h = Assert Pin High
						when  X"41" => local_dout <= data_ah; -- pin location
                                 ftdi_dout <= X"41";
						-- 50h = Read SRAM
						when  X"50" => local_dout <= local_din; -- read sram
                                 ftdi_dout <= local_din;
						-- 60h = Write SRAM
						when  X"60" => local_dout <= wdata; -- write sram
                                 ftdi_dout <= wdata;
						-- Any other value invalid command
						when others => local_dout <= X"E1"; -- command error
                                 ftdi_dout <= X"E1";
				   end case;
			   --end if;
			end if;
		end if;
	end process;

	SYNC_DIR : process (clk, reset) 
	begin
		if reset='1' then
			ftdi_dir <= '1'; --tristates enabled for input
		elsif clk='1' and clk'event then
			ftdi_dir <= NOT(send);
		end if;
	end process;

	WAIT_COUNT : process (clk) 
	begin
		if clk='1' and clk'event then
			if (reset='1' or wait_done='1') then 
				wcount <= (others => '0');
			elsif wait_start='1' then
				wcount <= wcount + 1;
			end if;
			if wcount(1 downto 0) = "11" then -- count = 0to3 + 1 clk delay = 5 periods
				wait_done <= '1';
			else
				wait_done <= '0';
			end if;
		end if;
	end process;

end Behavioral;

